searchData={"items":[{"type":"module","title":"Mox","doc":"Mox is a library for defining concurrent mocks in Elixir.\n\nThe library follows the principles outlined in\n[\"Mocks and explicit contracts\"](https://dashbit.co/blog/mocks-and-explicit-contracts),\nsummarized below:\n\n  1. No ad-hoc mocks. You can only create mocks based on behaviours\n\n  2. No dynamic generation of modules during tests. Mocks are preferably defined\n     in your `test_helper.exs` or in a `setup_all` block and not per test\n\n  3. Concurrency support. Tests using the same mock can still use `async: true`\n\n  4. Rely on pattern matching and function clauses for asserting on the\n     input instead of complex expectation rules","ref":"Mox.html"},{"type":"module","title":"Example - Mox","doc":"Imagine that you have an app that has to display the weather. At first,\nyou use an external API to give you the data given a lat/long pair:\n\n    defmodule MyApp.HumanizedWeather do\n      def display_temp({lat, long}) do\n        {:ok, temp} = MyApp.WeatherAPI.temp({lat, long})\n        \"Current temperature is #{temp} degrees\"\n      end\n\n      def display_humidity({lat, long}) do\n        {:ok, humidity} = MyApp.WeatherAPI.humidity({lat, long})\n        \"Current humidity is #{humidity}%\"\n      end\n    end\n\nHowever, you want to test the code above without performing external\nAPI calls. How to do so?\n\nFirst, it is important to define the `WeatherAPI` behaviour that we want\nto mock. And we will define a proxy function that will dispatch to\nthe desired implementation:\n\n    defmodule MyApp.WeatherAPI do\n      @callback temp(MyApp.LatLong.t()) :: {:ok, integer()}\n      @callback humidity(MyApp.LatLong.t()) :: {:ok, integer()}\n\n      def temp(lat_long), do: impl().temp(lat_long)\n      def humidity(lat_long), do: impl().humidity(lat_long)\n      defp impl, do: Application.get_env(:my_app, :weather, MyApp.ExternalWeatherAPI)\n    end\n\nBy default, we will dispatch to MyApp.ExternalWeatherAPI, which now contains\nthe external API implementation.\n\nIf you want to mock the WeatherAPI behaviour during tests, the first step\nis to define the mock with `defmock/2`, usually in your `test_helper.exs`,\nand configure your application to use it:\n\n    Mox.defmock(MyApp.MockWeatherAPI, for: MyApp.WeatherAPI)\n    Application.put_env(:my_app, :weather, MyApp.MockWeatherAPI)\n\nNow in your tests, you can define expectations with `expect/4` and verify\nthem via `verify_on_exit!/1`:\n\n    defmodule MyApp.HumanizedWeatherTest do\n      use ExUnit.Case, async: true\n\n      import Mox\n\n      # Make sure mocks are verified when the test exits\n      setup :verify_on_exit!\n\n      test \"gets and formats temperature and humidity\" do\n        MyApp.MockWeatherAPI\n        |> expect(:temp, fn {_lat, _long} -> {:ok, 30} end)\n        |> expect(:humidity, fn {_lat, _long} -> {:ok, 60} end)\n\n        assert MyApp.HumanizedWeather.display_temp({50.06, 19.94}) ==\n                 \"Current temperature is 30 degrees\"\n\n        assert MyApp.HumanizedWeather.display_humidity({50.06, 19.94}) ==\n                 \"Current humidity is 60%\"\n      end\n    end\n\nAll expectations are defined based on the current process. This\nmeans multiple tests using the same mock can still run concurrently\nunless the Mox is set to global mode. See the \"Multi-process collaboration\"\nsection.\n\nOne last note, if the mock is used throughout the test suite, you might want\nthe implementation to fall back to a stub (or actual) implementation when no\nexpectations are defined. You can use `stub_with/2` in a case template that\nis used throughout your test suite:\n\n    defmodule MyApp.Case do\n      use ExUnit.CaseTemplate\n\n      setup _ do\n        Mox.stub_with(MyApp.MockWeatherAPI, MyApp.StubWeatherAPI)\n        :ok\n      end\n    end\n\nNow, for every test case that uses `ExUnit.Case`, it can use `MyApp.Case`\ninstead. Then, if no expectations are defined it will call the implementation\nin `MyApp.StubWeatherAPI`.","ref":"Mox.html#module-example"},{"type":"module","title":"Multiple behaviours - Mox","doc":"Mox supports defining mocks for multiple behaviours.\n\nSuppose your library also defines a behaviour for getting past weather:\n\n    defmodule MyApp.PastWeather do\n      @callback past_temp(MyApp.LatLong.t(), DateTime.t()) :: {:ok, integer()}\n    end\n\nYou can mock both the weather and past weather behaviour:\n\n    Mox.defmock(MyApp.MockWeatherAPI, for: [MyApp.Weather, MyApp.PastWeather])","ref":"Mox.html#module-multiple-behaviours"},{"type":"module","title":"Compile-time requirements - Mox","doc":"If the mock needs to be available during the project compilation, for\ninstance because you get undefined function warnings, then instead of\ndefining the mock in your `test_helper.exs`, you should instead define\nit under `test/support/mocks.ex`:\n\n    Mox.defmock(MyApp.MockWeatherAPI, for: MyApp.WeatherAPI)\n\nThen you need to make sure that files in `test/support` get compiled\nwith the rest of the project. Edit your `mix.exs` file to add the\n`test/support` directory to compilation paths:\n\n    def project do\n      [\n        ...\n        elixirc_paths: elixirc_paths(Mix.env),\n        ...\n      ]\n    end\n\n    defp elixirc_paths(:test), do: [\"test/support\", \"lib\"]\n    defp elixirc_paths(_),     do: [\"lib\"]","ref":"Mox.html#module-compile-time-requirements"},{"type":"module","title":"Multi-process collaboration - Mox","doc":"Mox supports multi-process collaboration via two mechanisms:\n\n  1. explicit allowances\n  2. global mode\n\nThe allowance mechanism can still run tests concurrently while\nthe global one doesn't. We explore both next.","ref":"Mox.html#module-multi-process-collaboration"},{"type":"module","title":"Explicit allowances - Mox","doc":"An allowance permits a child process to use the expectations and stubs\ndefined in the parent process while still being safe for async tests.\n\n    test \"invokes add and mult from a task\" do\n      MyApp.MockWeatherAPI\n      |> expect(:temp, fn _loc -> {:ok, 30} end)\n      |> expect(:humidity, fn _loc -> {:ok, 60} end)\n\n      parent_pid = self()\n\n      Task.async(fn ->\n        MyApp.MockWeatherAPI |> allow(parent_pid, self())\n\n        assert MyApp.HumanizedWeather.display_temp({50.06, 19.94}) ==\n                 \"Current temperature is 30 degrees\"\n\n        assert MyApp.HumanizedWeather.display_humidity({50.06, 19.94}) ==\n                 \"Current humidity is 60%\"\n      end)\n      |> Task.await\n    end\n\nNote: if you're running on Elixir 1.8.0 or greater and your concurrency comes\nfrom a `Task` then you don't need to add explicit allowances. Instead\n`$callers` is used to determine the process that actually defined the\nexpectations.\n\n#### Explicit allowances as lazy/deferred functions\n\nUnder some circumstances, the process might not have been already started\nwhen the allowance happens. In such a case, you might specify the allowance\nas a function in the form `(-> pid())`. This function would be resolved late,\nat the very moment of dispatch. If the function does not return an existing\nPID, Mox will raise a `Mox.UnexpectedCallError` exception.","ref":"Mox.html#module-explicit-allowances"},{"type":"module","title":"Global mode - Mox","doc":"Mox supports global mode, where any process can consume mocks and stubs\ndefined in your tests. `set_mox_from_context/0` automatically calls\n`set_mox_global/1` but only if the test context **doesn't** include\n`async: true`.\n\nBy default the mode is `:private`.\n\n    setup :set_mox_from_context\n    setup :verify_on_exit!\n\n    test \"invokes add and mult from a task\" do\n      MyApp.MockWeatherAPI\n      |> expect(:temp, fn _loc -> {:ok, 30} end)\n      |> expect(:humidity, fn _loc -> {:ok, 60} end)\n\n      Task.async(fn ->\n        assert MyApp.HumanizedWeather.display_temp({50.06, 19.94}) ==\n                  \"Current temperature is 30 degrees\"\n\n        assert MyApp.HumanizedWeather.display_humidity({50.06, 19.94}) ==\n                  \"Current humidity is 60%\"\n      end)\n      |> Task.await\n    end","ref":"Mox.html#module-global-mode"},{"type":"module","title":"Blocking on expectations - Mox","doc":"If your mock is called in a different process than the test process,\nin some cases there is a chance that the test will finish executing\nbefore it has a chance to call the mock and meet the expectations.\nImagine this:\n\n    test \"calling a mock from a different process\" do\n      expect(MyApp.MockWeatherAPI, :temp, fn _loc -> {:ok, 30} end)\n\n      spawn(fn -> MyApp.HumanizedWeather.temp({50.06, 19.94}) end)\n\n      verify!()\n    end\n\nThe test above has a race condition because there is a chance that the\n`verify!/0` call will happen before the spawned process calls the mock.\nIn most cases, you don't control the spawning of the process so you can't\nsimply monitor the process to know when it dies in order to avoid this\nrace condition. In those cases, the way to go is to \"sync up\" with the\nprocess that calls the mock by sending a message to the test process\nfrom the expectation and using that to know when the expectation has been\ncalled.\n\n    test \"calling a mock from a different process\" do\n      parent = self()\n      ref = make_ref()\n\n      expect(MyApp.MockWeatherAPI, :temp, fn _loc ->\n        send(parent, {ref, :temp})\n        {:ok, 30}\n      end)\n\n      spawn(fn -> MyApp.HumanizedWeather.temp({50.06, 19.94}) end)\n\n      assert_receive {^ref, :temp}\n\n      verify!()\n    end\n\nThis way, we'll wait until the expectation is called before calling\n`verify!/0`.","ref":"Mox.html#module-blocking-on-expectations"},{"type":"function","title":"Mox.allow/3","doc":"Allows other processes to share expectations and stubs\ndefined by owner process.","ref":"Mox.html#allow/3"},{"type":"function","title":"Examples - Mox.allow/3","doc":"To allow `child_pid` to call any stubs or expectations defined for `MyMock`:\n\n    allow(MyMock, self(), child_pid)\n\n`allow/3` also accepts named process or via references:\n\n    allow(MyMock, self(), SomeChildProcess)\n\nIf the process is not yet started at the moment of allowance definition,\nit might be allowed as a function, assuming at the moment of invocation\nit would have been started. If the function cannot be resolved to a `pid`\nduring invocation, the expectation will not succeed.\n\n    allow(MyMock, self(), fn -> GenServer.whereis(Deferred) end)","ref":"Mox.html#allow/3-examples"},{"type":"function","title":"Mox.defmock/2","doc":"Defines a mock with the given name `:for` the given behaviour(s).\n\n    Mox.defmock(MyMock, for: MyBehaviour)\n\nWith multiple behaviours:\n\n    Mox.defmock(MyMock, for: [MyBehaviour, MyOtherBehaviour])","ref":"Mox.html#defmock/2"},{"type":"function","title":"Options - Mox.defmock/2","doc":"* `:for` - module or list of modules to define the mock module for.\n\n  * `:moduledoc` - `@moduledoc` for the defined mock module.\n\n  * `:skip_optional_callbacks` - boolean to determine whether to skip\n    or generate optional callbacks in the mock module.","ref":"Mox.html#defmock/2-options"},{"type":"function","title":"Skipping optional callbacks - Mox.defmock/2","doc":"By default, functions are created for all the behaviour's callbacks,\nincluding optional ones. But if for some reason you want to skip one or more\nof its `@optional_callbacks`, you can provide the list of callback names to\nskip (along with their arities) as `:skip_optional_callbacks`:\n\n    Mox.defmock(MyMock, for: MyBehaviour, skip_optional_callbacks: [on_success: 2])\n\nThis will define a new mock (`MyMock`) that has a defined function for each\ncallback on `MyBehaviour` except for `on_success/2`. Note: you can only skip\noptional callbacks, not required callbacks.\n\nYou can also pass `true` to skip all optional callbacks, or `false` to keep\nthe default of generating functions for all optional callbacks.","ref":"Mox.html#defmock/2-skipping-optional-callbacks"},{"type":"function","title":"Passing `@moduledoc` - Mox.defmock/2","doc":"You can provide value for `@moduledoc` with `:moduledoc` option.\n\n    Mox.defmock(MyMock, for: MyBehaviour, moduledoc: false)\n    Mox.defmock(MyMock, for: MyBehaviour, moduledoc: \"My mock module.\")","ref":"Mox.html#defmock/2-passing-moduledoc"},{"type":"function","title":"Mox.deny/3","doc":"Ensures that `name`/`arity` in `mock` is not invoked.\n\nWhen `deny/3` is invoked, any previously declared `stub` for the same `name` and arity will\nbe removed. This ensures that `deny` will fail if the function is called. If a `stub/3` is\ninvoked **after** `deny/3` for the same `name` and `arity`, the stub will be used instead, so\n`deny` will have no effect.","ref":"Mox.html#deny/3"},{"type":"function","title":"Examples - Mox.deny/3","doc":"To expect `MockWeatherAPI.get_temp/1` to never be called:\n\n    deny(MockWeatherAPI, :get_temp, 1)","ref":"Mox.html#deny/3-examples"},{"type":"function","title":"Mox.expect/4","doc":"Expects the `name` in `mock` with arity given by `code`\nto be invoked `n` times.\n\nIf you're calling your mock from an asynchronous process and want\nto wait for the mock to be called, see the \"Blocking on expectations\"\nsection in the module documentation.\n\nWhen `expect/4` is invoked, any previously declared `stub` for the same `name` and arity will\nbe removed. This ensures that `expect` will fail if the function is called more than `n` times.\nIf a `stub/3` is invoked **after** `expect/4` for the same `name` and arity, the stub will be\nused after all expectations are fulfilled.","ref":"Mox.html#expect/4"},{"type":"function","title":"Examples - Mox.expect/4","doc":"To expect `MockWeatherAPI.get_temp/1` to be called once:\n\n    expect(MockWeatherAPI, :get_temp, fn _ -> {:ok, 30} end)\n\nTo expect `MockWeatherAPI.get_temp/1` to be called five times:\n\n    expect(MockWeatherAPI, :get_temp, 5, fn _ -> {:ok, 30} end)\n\nTo expect `MockWeatherAPI.get_temp/1` not to be called (see also `deny/3`):\n\n    expect(MockWeatherAPI, :get_temp, 0, fn _ -> {:ok, 30} end)\n\n`expect/4` can also be invoked multiple times for the same name/arity,\nallowing you to give different behaviours on each invocation. For instance,\nyou could test that your code will try an API call three times before giving\nup:\n\n    MockWeatherAPI\n    |> expect(:get_temp, 2, fn _loc -> {:error, :unreachable} end)\n    |> expect(:get_temp, 1, fn _loc -> {:ok, 30} end)\n\n    log = capture_log(fn ->\n      assert Weather.current_temp(location)\n        == \"It's currently 30 degrees\"\n    end)\n\n    assert log =~ \"attempt 1 failed\"\n    assert log =~ \"attempt 2 failed\"\n    assert log =~ \"attempt 3 succeeded\"\n\n    MockWeatherAPI\n    |> expect(:get_temp, 3, fn _loc -> {:error, :unreachable} end)\n\n    assert Weather.current_temp(location) == \"Current temperature is unavailable\"","ref":"Mox.html#expect/4-examples"},{"type":"function","title":"Mox.set_mox_from_context/1","doc":"Chooses the Mox mode based on context.\n\nWhen `async: true` is used, `set_mox_private/1` is called,\notherwise `set_mox_global/1` is used.","ref":"Mox.html#set_mox_from_context/1"},{"type":"function","title":"Examples - Mox.set_mox_from_context/1","doc":"setup :set_mox_from_context","ref":"Mox.html#set_mox_from_context/1-examples"},{"type":"function","title":"Mox.set_mox_global/1","doc":"Sets the Mox to global mode.\n\nIn global mode, mocks can be consumed by any process.\n\nAn ExUnit case where tests use Mox in global mode cannot be\n`async: true`.","ref":"Mox.html#set_mox_global/1"},{"type":"function","title":"Examples - Mox.set_mox_global/1","doc":"setup :set_mox_global","ref":"Mox.html#set_mox_global/1-examples"},{"type":"function","title":"Mox.set_mox_private/1","doc":"Sets the Mox to private mode.\n\nIn private mode, mocks can be set and consumed by the same\nprocess unless other processes are explicitly allowed.","ref":"Mox.html#set_mox_private/1"},{"type":"function","title":"Examples - Mox.set_mox_private/1","doc":"setup :set_mox_private","ref":"Mox.html#set_mox_private/1-examples"},{"type":"function","title":"Mox.stub/3","doc":"Allows the `name` in `mock` with arity given by `code` to\nbe invoked zero or many times.\n\nUnlike expectations, stubs are never verified.\n\nIf expectations and stubs are defined for the same function\nand arity, the stub is invoked only after all expectations are\nfulfilled.","ref":"Mox.html#stub/3"},{"type":"function","title":"Examples - Mox.stub/3","doc":"To allow `MockWeatherAPI.get_temp/1` to be called any number of times:\n\n    stub(MockWeatherAPI, :get_temp, fn _loc -> {:ok, 30} end)\n\n`stub/3` will overwrite any previous calls to `stub/3`.","ref":"Mox.html#stub/3-examples"},{"type":"function","title":"Mox.stub_with/2","doc":"Stubs all functions described by the shared behaviours in the `mock` and `module`.","ref":"Mox.html#stub_with/2"},{"type":"function","title":"Examples - Mox.stub_with/2","doc":"defmodule MyApp.WeatherAPI do\n      @callback temp(MyApp.LatLong.t()) :: {:ok, integer()}\n      @callback humidity(MyApp.LatLong.t()) :: {:ok, integer()}\n    end\n\n    defmodule MyApp.StubWeatherAPI do\n      @behaviour MyApp.WeatherAPI\n      def temp(_loc), do: {:ok, 30}\n      def humidity(_loc), do: {:ok, 60}\n    end\n\n    defmock(MyApp.MockWeatherAPI, for: MyApp.WeatherAPI)\n\n    setup do\n      stub_with(MyApp.MockWeatherAPI, MyApp.StubWeatherAPI)\n      :ok\n    end\n\nThis is the same as calling `stub/3` for each callback in `MyApp.MockWeatherAPI`:\n\n    stub(MyApp.MockWeatherAPI, :temp, &MyApp.StubWeatherAPI.temp/1)\n    stub(MyApp.MockWeatherAPI, :humidity, &MyApp.StubWeatherAPI.humidity/1)","ref":"Mox.html#stub_with/2-examples"},{"type":"function","title":"Mox.verify!/0","doc":"Verifies that all expectations set by the current process\nhave been called.","ref":"Mox.html#verify!/0"},{"type":"function","title":"Mox.verify!/1","doc":"Verifies that all expectations in `mock` have been called.","ref":"Mox.html#verify!/1"},{"type":"function","title":"Mox.verify_on_exit!/1","doc":"Verifies the current process after it exits.\n\nIf you want to verify expectations for all tests, you can use\n`verify_on_exit!/1` as a setup callback:\n\n    setup :verify_on_exit!","ref":"Mox.html#verify_on_exit!/1"},{"type":"type","title":"Mox.t/0","doc":"A mock module.\n\nThis type is available since version 1.1+ of Mox.","ref":"Mox.html#t:t/0"},{"type":"exception","title":"Mox.UnexpectedCallError","doc":"","ref":"Mox.UnexpectedCallError.html"},{"type":"exception","title":"Mox.VerificationError","doc":"","ref":"Mox.VerificationError.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}